@@ -35,6 +35,8 @@
 #define STYLE_SECONDARY_CURSOR		"secondary-cursor"
 #define STYLE_CURRENT_LINE		"current-line"
 #define STYLE_LINE_NUMBERS		"line-numbers"
+#define STYLE_RIGHT_MARGIN		"right-margin"
+#define STYLE_DRAW_SPACES		"draw-spaces"
 
 #define STYLE_SCHEME_VERSION		"1.0"
 
@@ -74,13 +76,13 @@ gtk_source_style_scheme_finalize (GObject *object)
 	g_hash_table_destroy (scheme->priv->style_cache);
 	g_hash_table_destroy (scheme->priv->defined_styles);
 	g_free (scheme->priv->filename);
-	
-	if (scheme->priv->authors != NULL) 
+
+	if (scheme->priv->authors != NULL)
 	{
 		g_ptr_array_foreach (scheme->priv->authors, (GFunc)g_free, NULL);
 		g_ptr_array_free (scheme->priv->authors, TRUE);
 	}
-	
+
 	g_free (scheme->priv->description);
 	g_free (scheme->priv->id);
 	g_free (scheme->priv->name);
@@ -189,7 +191,7 @@ gtk_source_style_scheme_class_init (GtkSourceStyleSchemeClass *klass)
 	 * Style scheme name, a translatable string to present to user.
 	 */
 	g_object_class_install_property (object_class,
-					 PROP_NAME,
+					 PROP_DESCRIPTION,
 					 g_param_spec_string ("description",
 						 	      _("Style scheme description"),
 							      _("Style scheme description"),
@@ -245,6 +247,7 @@ gtk_source_style_scheme_get_id (GtkSourceStyleScheme *scheme)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme), NULL);
 	g_return_val_if_fail (scheme->priv->id != NULL, "");
+
 	return scheme->priv->id;
 }
 
@@ -261,6 +264,7 @@ gtk_source_style_scheme_get_name (GtkSourceStyleScheme *scheme)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme), NULL);
 	g_return_val_if_fail (scheme->priv->name != NULL, "");
+
 	return scheme->priv->name;
 }
 
@@ -276,6 +280,7 @@ const gchar *
 gtk_source_style_scheme_get_description (GtkSourceStyleScheme *scheme)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme), NULL);
+
 	return scheme->priv->description;
 }
 
@@ -283,7 +288,7 @@ gtk_source_style_scheme_get_description (GtkSourceStyleScheme *scheme)
  * gtk_source_style_scheme_get_authors:
  * @scheme: a #GtkSourceStyleScheme.
  *
- * Returns: a %NULL-terminated array containing the @scheme authors or 
+ * Returns: a %NULL-terminated array containing the @scheme authors or
  * %NULL if no author is specified by the style
  * scheme.
  *
@@ -313,6 +318,7 @@ const gchar *
 gtk_source_style_scheme_get_filename (GtkSourceStyleScheme *scheme)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme), NULL);
+
 	return scheme->priv->filename;
 }
 
@@ -508,9 +514,26 @@ GtkSourceStyle *
 _gtk_source_style_scheme_get_matching_brackets_style (GtkSourceStyleScheme *scheme)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme), NULL);
+
 	return gtk_source_style_scheme_get_style (scheme, STYLE_BRACKET_MATCH);
 }
 
+GtkSourceStyle *
+_gtk_source_style_scheme_get_right_margin_style (GtkSourceStyleScheme *scheme)
+{
+	g_return_val_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme), NULL);
+
+	return gtk_source_style_scheme_get_style (scheme, STYLE_RIGHT_MARGIN);
+}
+
+GtkSourceStyle *
+_gtk_source_style_scheme_get_draw_spaces_style (GtkSourceStyleScheme *scheme)
+{
+	g_return_val_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme), NULL);
+
+	return gtk_source_style_scheme_get_style (scheme, STYLE_DRAW_SPACES);
+}
+
 static gboolean
 get_color (GtkSourceStyle *style,
 	   gboolean        foreground,
@@ -640,6 +663,10 @@ set_cursor_colors (GtkWidget      *widget,
 	if (strcmp (widget_name, gtk_widget_get_name (widget)) != 0)
 		gtk_widget_set_name (widget, widget_name);
 
+	g_object_set_data (G_OBJECT (widget),
+			   "gtk-source-view-cursor-color-set",
+			   GINT_TO_POINTER (TRUE));
+
 	g_free (rc_string);
 	g_free (widget_name);
 #else
@@ -651,9 +678,10 @@ static void
 unset_cursor_colors (GtkWidget *widget)
 {
 #if !GTK_CHECK_VERSION(2,11,3)
-	set_cursor_colors (widget,
-			   &widget->style->text[GTK_STATE_NORMAL],
-			   &widget->style->text_aa[GTK_STATE_NORMAL]);
+	if (g_object_get_data (G_OBJECT (widget), "gtk-source-view-cursor-color-set") != NULL)
+		set_cursor_colors (widget,
+				   &widget->style->text[GTK_STATE_NORMAL],
+				   &widget->style->text_aa[GTK_STATE_NORMAL]);
 #else
 	gtk_widget_modify_cursor (widget, NULL, NULL);
 #endif
@@ -775,7 +803,9 @@ parse_style (GtkSourceStyleScheme *scheme,
 {
 	GtkSourceStyle *use_style = NULL;
 	GtkSourceStyle *result = NULL;
-	xmlChar *fg = NULL, *bg = NULL, *line_bg = NULL;
+	xmlChar *fg = NULL;
+	xmlChar *bg = NULL;
+	xmlChar *line_bg = NULL;
 	gchar *style_name = NULL;
 	guint mask = 0;
 	gboolean bold = FALSE;
@@ -840,6 +870,7 @@ parse_style (GtkSourceStyleScheme *scheme,
 			g_free (style_name);
 			xmlFree (fg);
 			xmlFree (bg);
+			xmlFree (line_bg);
 			return FALSE;
 		}
 
@@ -878,6 +909,7 @@ parse_style (GtkSourceStyleScheme *scheme,
 	*style_p = result;
 	*style_name_p = style_name;
 
+	xmlFree (line_bg);
 	xmlFree (bg);
 	xmlFree (fg);
 
@@ -944,7 +976,7 @@ parse_style_scheme_child (GtkSourceStyleScheme *scheme,
 			scheme->priv->authors = g_ptr_array_new ();
 
 		g_ptr_array_add (scheme->priv->authors, g_strdup ((char*) tmp));
-		
+
 		xmlFree (tmp);
 	}
 	else if (strcmp ((char*) node->name, "description") == 0)
@@ -1028,7 +1060,7 @@ parse_style_scheme_element (GtkSourceStyleScheme *scheme,
 		if (node->type == XML_ELEMENT_NODE)
 			if (!parse_style_scheme_child (scheme, node, error))
 				return;
-				
+
 	/* NULL-terminate the array of authors */
 	if (scheme->priv->authors != NULL)
 		g_ptr_array_add (scheme->priv->authors, NULL);
